@@ -21,12 +21,14 @@ from conda.misc import clone_env, explicit, touch_nonadmin
 from conda.common.serialize import json_dump
 from conda.cli.common import specs_from_url, confirm_yn, check_non_admin, ensure_name_or_prefix
 from conda.core.subdir_data import SubdirData
+from conda.history import History
 from conda.core.link import UnlinkLinkTransaction, PrefixSetup
 from conda.cli.install import check_prefix, clone, print_activate
 from conda.base.constants import ChannelPriority, ROOT_ENV_NAME, UpdateModifier, DepsModifier
-from conda.core.solve import diff_for_unlink_link_precs
+from conda.core.solve import diff_for_unlink_link_precs, get_pinned_specs
 from conda.core.envs_manager import unregister_env
 from conda.core.package_cache_data import PackageCacheData
+from conda.common.compat import on_win
 
 # create support
 from conda.common.path import paths_equal
@@ -55,7 +57,7 @@ import mamba.mamba_api as api
 from mamba.utils import get_index, to_package_record_from_subjson, init_api_context
 
 log = getLogger(__name__)
-stderrlog = getLogger('mamba.stderr')
+stderrlog = getLogger('conda.stderr')
 
 banner = f"""
                   __    __    __    __
@@ -129,16 +131,15 @@ def specs_from_args(args, json=False):
     return [arg2spec(arg, json=json) for arg in args]
 
 
-def to_txn(specs_to_add, specs_to_remove, prefix, to_link, to_unlink, index=None):
+def to_txn(specs_to_add, specs_to_remove, prefix, to_link, to_unlink, index=[]):
     to_link_records, to_unlink_records = [], []
 
     prefix_data = PrefixData(prefix)
     final_precs = IndexedSet(prefix_data.iter_records())
 
-    def get_channel(c):
-        for _, chan in index:
-            if str(chan) == c:
-                return chan
+    lookup_dict = {}
+    for _, c in index:
+        lookup_dict[c.url(with_credentials=True)] = c
 
     for c, pkg in to_unlink:
         for i_rec in installed_pkg_recs:
@@ -150,7 +151,7 @@ def to_txn(specs_to_add, specs_to_remove, prefix, to_link, to_unlink, index=None
             print("No package record found!")
 
     for c, pkg, jsn_s in to_link:
-        sdir = get_channel(c)
+        sdir = lookup_dict[c]
         rec = to_package_record_from_subjson(sdir, pkg, jsn_s)
         final_precs.add(rec)
         to_link_records.append(rec)
@@ -329,6 +330,7 @@ def install(args, parser, command='install'):
     isupdate = bool(command == 'update')
     if isupdate:
         solver_task = api.SOLVER_UPDATE
+        solver_options.clear()
 
     if newenv:
         ensure_name_or_prefix(args, command)
@@ -347,7 +349,9 @@ def install(args, parser, command='install'):
 
     if not newenv:
         if isdir(prefix):
-            delete_trash(prefix)
+            if on_win:
+                delete_trash(prefix)
+
             if not isfile(join(prefix, 'conda-meta', 'history')):
                 if paths_equal(prefix, context.conda_prefix):
                     raise NoBaseEnvironmentError()
@@ -393,6 +397,14 @@ def install(args, parser, command='install'):
 
     specs = []
 
+    index_args = {
+        'use_cache': args.use_index_cache,
+        'channel_urls': context.channels,
+        'unknown': args.unknown,
+        'prepend': not args.override_channels,
+        'use_local': args.use_local
+    }
+
     if args.file:
         file_specs = []
         for fpath in args.file:
@@ -408,6 +420,7 @@ def install(args, parser, command='install'):
 
     specs.extend(specs_from_args(args_packages, json=context.json))
 
+    # update channels from package specs (e.g. mychannel::mypackage adds mychannel)
     channels = [c for c in context.channels]
     for spec in specs:
         # CONDA TODO: correct handling for subdir isn't yet done
@@ -415,13 +428,7 @@ def install(args, parser, command='install'):
         if spec_channel and spec_channel not in channels:
             channels.append(spec_channel)
 
-    index_args = {
-        'use_cache': args.use_index_cache,
-        'channel_urls': channels,
-        'unknown': args.unknown,
-        'prepend': not args.override_channels,
-        'use_local': args.use_local
-    }
+    index_args['channel_urls'] = channels
 
     index = get_index(channel_urls=index_args['channel_urls'],
                       prepend=index_args['prepend'], platform=None,
@@ -430,7 +437,7 @@ def install(args, parser, command='install'):
 
     channel_json = []
     strict_priority = (context.channel_priority == ChannelPriority.STRICT)
-
+    subprio_index = len(index)
     if strict_priority:
         # first, count unique channels
         n_channels = len(set([channel.canonical_name for _, channel in index]))
@@ -446,16 +453,21 @@ def install(args, parser, command='install'):
             priority = channel_prio
         else:
             priority = 0
-
-        subpriority = 0 if chan.platform == 'noarch' else 1
+        if strict_priority:
+            subpriority = 0 if chan.platform == 'noarch' else 1
+        else:
+            subpriority = subprio_index
+            subprio_index -= 1
 
         if subdir.loaded() == False and chan.platform != 'noarch':
             # ignore non-loaded subdir if channel is != noarch
             continue
 
         if context.verbosity != 0:
+            print("Channel: {}, prio: {} : {}".format(chan, priority, subpriority))
             print("Cache path: ", subdir.cache_path())
-        channel_json.append((chan, subdir.cache_path(), priority, subpriority))
+
+        channel_json.append((chan, subdir, priority, subpriority))
 
     # for c in channel_json:
     #     print(c[0], c[2], c[3])
@@ -468,14 +480,28 @@ def install(args, parser, command='install'):
         raise CondaValueError("too few arguments, "
                               "must supply command line package specs or --file")
 
+    installed_names = [i_rec.name for i_rec in installed_pkg_recs]
     # for 'conda update', make sure the requested specs actually exist in the prefix
     # and that they are name-only specs
     if isupdate and context.update_modifier == UpdateModifier.UPDATE_ALL:
-        # Note: History(prefix).get_requested_specs_map()
-        print("Currently, mamba can only update explicit packages! (e.g. mamba update numpy python ...)")
-        exit()
+        history_dict = History(prefix).get_requested_specs_map()
+        pins = get_pinned_specs(prefix)
+        pin_names = [p.name for p in pins]
+
+        # for key, match_spec in history_dict.items():
+        for key in installed_names:
+
+            if key == 'python':
+                i = installed_names.index('python')
+                version = installed_pkg_recs[i].version
+                py_ver = ".".join(version.split(".")[:2]) + '.*'
+                # specs.append(MatchSpec(name="python", version=py_ver))
+            else:
+                if key in pin_names:
+                    specs.append(pins[key])
+                else:
+                    specs.append(MatchSpec(key))
 
-    if isupdate and context.update_modifier != UpdateModifier.UPDATE_ALL:
         prefix_data = PrefixData(prefix)
         for s in args_packages:
             s = MatchSpec(s)
@@ -485,6 +511,18 @@ def install(args, parser, command='install'):
             if not prefix_data.get(s.name, None):
                 raise PackageNotInstalledError(prefix, s.name)
 
+    elif context.update_modifier == UpdateModifier.UPDATE_DEPS:
+        # find the deps for each package and add to the update job
+        # solver_task |= api.SOLVER_FORCEBEST
+        final_specs = specs
+        for spec in specs:
+            prec = installed_pkg_recs[installed_names.index(spec.name)]
+            for dep in prec.depends:
+                ms = MatchSpec(dep)
+                if ms.name != 'python':
+                    final_specs.append(MatchSpec(ms.name))
+        specs = set(final_specs)
+
     if newenv and args.clone:
         if args.packages:
             raise TooManyArgumentsError(0, len(args.packages), list(args.packages),
@@ -495,16 +533,18 @@ def install(args, parser, command='install'):
         print_activate(args.name if args.name else prefix)
         return
 
-    spec_names = [s.name for s in specs]
 
     if not (context.quiet or context.json):
-        print("\nLooking for: {}\n".format(spec_names))
+        print("\nLooking for: {}\n".format([str(s) for s in specs]))
+
+    spec_names = [s.name for s in specs]
 
     # If python was not specified, check if it is installed.
     # If yes, add the installed python to the specs to prevent updating it.
     python_constraint = None
+    additional_specs = []
+
     if 'python' not in spec_names:
-        installed_names = [i_rec.name for i_rec in installed_pkg_recs]
         if 'python' in installed_names:
             i = installed_names.index('python')
             version = installed_pkg_recs[i].version
@@ -516,10 +556,12 @@ def install(args, parser, command='install'):
 
     repos = []
 
-    # add installed
-    if use_mamba_experimental:
+    if use_mamba_experimental or context.force_reinstall:
         prefix_data = api.PrefixData(context.target_prefix)
         prefix_data.load()
+
+    # add installed
+    if use_mamba_experimental:
         repo = api.Repo(pool, prefix_data)
         repos.append(repo)
     else:
@@ -527,21 +569,31 @@ def install(args, parser, command='install'):
         repo.set_installed()
         repos.append(repo)
 
-    for channel, cache_file, priority, subpriority in channel_json:
-        repo = api.Repo(pool, str(channel), cache_file, channel.url(with_credentials=True))
+    for channel, subdir, priority, subpriority in channel_json:
+        repo = subdir.create_repo(pool)
         repo.set_priority(priority, subpriority)
         repos.append(repo)
 
-    solver = api.Solver(pool, solver_options)
+    if context.force_reinstall:
+        solver = api.Solver(pool, solver_options, prefix_data)
+    else:
+        solver = api.Solver(pool, solver_options)
+
+    solver.set_postsolve_flags(
+        [(api.MAMBA_NO_DEPS, context.deps_modifier == DepsModifier.NO_DEPS), 
+         (api.MAMBA_ONLY_DEPS, context.deps_modifier == DepsModifier.ONLY_DEPS),
+         (api.MAMBA_FORCE_REINSTALL, context.force_reinstall)]
+    )
     solver.add_jobs(mamba_solve_specs, solver_task)
 
+    # as a security feature this will _always_ attempt to upgrade certain packages
+    for a_pkg in [_.name for _ in context.aggressive_update_packages]:
+        if a_pkg in installed_names:
+            solver.add_jobs([a_pkg], api.SOLVER_UPDATE)
+
     if python_constraint:
         solver.add_constraint(python_constraint)
 
-    solver.set_postsolve_flags(
-        [(api.MAMBA_NO_DEPS, context.deps_modifier == DepsModifier.NO_DEPS), 
-         (api.MAMBA_ONLY_DEPS, context.deps_modifier == DepsModifier.ONLY_DEPS)]
-    )
     success = solver.solve()
     if not success:
         print(solver.problems_to_str())
@@ -632,6 +684,8 @@ def repoquery(args, parser):
     only_installed = True
     if args.subcmd == "search" and args.installed == False:
         only_installed = False
+    elif args.all_channels:
+        only_installed = False
 
     if not only_installed:
         index = get_index(channel_urls=index_args['channel_urls'],
@@ -715,6 +769,12 @@ Examples:
         help=SUPPRESS
     )
 
+    package_cmds.add_argument(
+        "-a", "--all-channels",
+        action="store_true",
+        help="Look at all channels (for depends / whoneeds)"
+    )
+
     view_cmds = argparse.ArgumentParser(add_help=False)
     view_cmds.add_argument(
         "-t", "--tree",
